# Copyright (C) 2008 Red Hat, Inc

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# a long with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# Copyright (C) 2008 Red Hat, Inc

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# a long with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
require 'git'
require 'genome-lib'
require 'genome-sync'

module GenomeSync
  class Saver
    include ClassAccessorsWithDefaults
    include Loggable

    cattr_accessor :puppetmaster_tracking_branch, "master"

    def initialize(local_path, working_path)
      # The local git-dir
      @local_path = File.expand_path(local_path)
      @local_repo_parent_dir, @local_repo_name = File.split(@local_path)

      @working_path = File.expand_path(working_path)
    end

    def save
      unless File.directory?(@local_path)
        log("Making #{@local_repo_parent_dir}")
        FileUtils.mkdir_p(@local_repo_parent_dir)

        do_local_clone
      end

      push_content
    end

    private
    
    def puppet_repo?
      @local_repo_parent_dir =~ %r-puppet- 
    end

    def push_content
      git = Git.open(@working_path)
      cb = git.current_branch
      log("Current branch is #{cb}")

      log("Pushing all branches in #{@working_path} to #{@local_path}:")
      git.branches.local.each do |b| 
        next if b.name == "(no branch)" #important for superprojects
        b.checkout
        log("#{b}")
        git.push(@local_path, b, :force => true)
      end

      log("Switching back to previous current branch")
      git.checkout(cb)
      log("done.")
    end

    def do_local_clone
      log("Cloning #{@working_path} to #{@local_path}")
      # Handle the special puppet module case
      if puppet_repo?
        # For the correct code to make it to the puppetmaster we need to
        # make sure we checkout the desired tracking branch.
        Git.open(@working_path).checkout(puppetmaster_tracking_branch)

        log("Checking out '#{puppetmaster_tracking_branch}' in the Puppetmaster working dir")
        Git.clone(@working_path, 
                  @local_repo_name,
                  :path => @local_repo_parent_dir)
      else 
        Git.clone(@working_path, 
                  @local_repo_name, 
                  :path => @local_repo_parent_dir,
                  :bare => true)
      end
    end
  end
end
